/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.stub;


import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.testbase.knowledge.basis.GeneratorResult;
import ru.ispras.testbase.knowledge.floating.FpException;
import ru.ispras.testbase.knowledge.floating.FpExceptionGenerator;
import ru.ispras.testbase.knowledge.floating.FpNumber;
import ru.ispras.testbase.generator.DataGeneratorBase;
import ru.ispras.testbase.generator.DataGeneratorQuery;
import ru.ispras.testbase.generator.DataGeneratorVariable;

abstract class TdpFpBase extends DataGeneratorBase {
  protected TdpFpBase(final String name) {
    super(name);
  }

  protected abstract FpExceptionGenerator newFpGenerator(
      final int exponentLength, final int fractionLength);

  @Override
  protected void generate(final DataGeneratorQuery query) {
    final int exp = query.getIntegerParameter("exp").intValue();
    final int frac = query.getIntegerParameter("frac").intValue();

    final Object caseParam = query.getParameter("case");
    final FpException caseId = FpException.get(caseParam.toString());

    final FpNumber[] numbers = new FpNumber[2];
    final GeneratorResult<FpNumber> result = newFpGenerator(exp, frac).generate(caseId, numbers);

    int dataIndex = 0;
    for (final DataGeneratorVariable variable : query.getUnknownVariables()) {
      if (dataIndex >= result.operands.length) {
        break;
      }

      if (!variable.getName().endsWith("fd")) {
        final FpNumber fpData = result.operands[dataIndex];

        final BitVector bvSign = BitVector.valueOf(fpData.getSign(), 1);
        final BitVector bvExp = BitVector.valueOf(fpData.getExponent(), exp);
        final BitVector bvFrac = BitVector.valueOf(fpData.getFraction(), frac);

        final BitVector value = BitVector.newMapping(bvFrac, bvExp, bvSign);
        variable.setValue(value.copy());

        dataIndex++;
      }
    }
  }
}
